@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from types import TracebackType
-from typing import Any, AnyStr, Awaitable, List, Optional, Tuple
+from typing import Any, AnyStr, Awaitable, List, Optional, Tuple, Union
 
 import trio
 from hypercorn.typing import HTTPScope, WebsocketScope
@@ -27,8 +27,10 @@ class TestHTTPConnection:
         self.scope = scope
         self.status_code: Optional[int] = None
         self._preserve_context = _preserve_context
-        self._server_send, self._server_receive = trio.open_memory_channel(10)
-        self._client_send, self._client_receive = trio.open_memory_channel(10)
+        self._server_send, self._server_receive = trio.open_memory_channel[dict](10)
+        self._client_send, self._client_receive = trio.open_memory_channel[Union[bytes, Exception]](
+            10
+        )
 
     async def send(self, data: bytes) -> None:
         await self._server_send.send({"type": "http.request", "body": data, "more_body": True})
@@ -50,7 +52,7 @@ class TestHTTPConnection:
     async def __aenter__(self) -> TestHTTPConnectionProtocol:
         self._nursery_manager = trio.open_nursery()
         nursery = await self._nursery_manager.__aenter__()
-        nursery.start_soon(self.app, self.scope, self._asgi_receive, self._asgi_send)
+        nursery.start_soon(self.app, self.scope, self._asgi_receive, self._asgi_send)  # type: ignore  # noqa: E501
         return self
 
     async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
@@ -103,14 +105,16 @@ class TestWebsocketConnection:
         self.response_data = bytearray()
         self.scope = scope
         self.status_code: Optional[int] = None
-        self._server_send, self._server_receive = trio.open_memory_channel(10)
-        self._client_send, self._client_receive = trio.open_memory_channel(10)
+        self._server_send, self._server_receive = trio.open_memory_channel[dict](10)
+        self._client_send, self._client_receive = trio.open_memory_channel[
+            Union[AnyStr, Exception]
+        ](10)
         self._task: Awaitable[None] = None
 
     async def __aenter__(self) -> TestWebsocketConnectionProtocol:
         self._nursery_manager = trio.open_nursery()
         nursery = await self._nursery_manager.__aenter__()
-        nursery.start_soon(self.app, self.scope, self._asgi_receive, self._asgi_send)
+        nursery.start_soon(self.app, self.scope, self._asgi_receive, self._asgi_send)  # type: ignore # noqa: E501
         return self
 
     async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
